__package_name__ = 'atcoder-submit-status'
__author__ = 'ngng628'
__email__ = 'ngng628.kyopro@gmail.com'
__license__ = 'MIT License'
__url__ = 'https://github.com/ngng628/atcoder-submit-status'
__version_info__ = (0, 0, 4)
__version__ = '.'.join(map(str, __version_info__[:3]))
__description__ = 'CLI tool to fetch atcoder-submit-status'